#include "server/auth.h"
#include "server/response.h"
#include "server/util.h"

#include "db/auth.h"

#include "hash/hash.h"

#include <civetweb.h>

#include <cjson/cJSON.h>

#include <assert.h>
#include <stdlib.h>
#include <string.h>

typedef struct
{
    char* password;
} admin_form_t;

static void admin_form_dtor(admin_form_t* form)
{
    if (form->password) free(form->password);
}

static int field_found(const char* key, const char* filename, char* path, size_t pathlen, void* user_data)
{
    return MG_FORM_FIELD_HANDLE_GET;
}

static int field_get(const char* key, const char* value, size_t valuelen, void* user_data)
{
    admin_form_t* form = (admin_form_t*)user_data;
    if (strcmp(key, "password") == 0) {
        form->password = kqm_strndup(value, valuelen);
        return MG_FORM_FIELD_HANDLE_ABORT;
    }
    return MG_FORM_FIELD_HANDLE_GET;
}

int admin_handler(mg_connection* conn, void* cbdata)
{
    const mg_request_info* post_body = mg_get_request_info(conn);

    if (post_body == NULL) {
        res_null_req(conn);
        return 1;
    }

    if (strcmp(post_body->request_method, "POST")) {
        res_must_post(conn);
        return 1;
    }

    admin_form_t form = {NULL};

    mg_form_data_handler admin_callback = {
        .field_found = field_found,
        .field_get   = field_get,
        .field_store = NULL,
        .user_data   = &form,
    };

    mg_handle_form_request(conn, &admin_callback);

    if (!form.password) {
        res_need_password(conn);
        admin_form_dtor(&form);
        return 1;
    }

    int result;
    int flag = admin_login(form.password, &result);
    if (!flag) {
        res_login_fail(conn);
    } else if (!result) {
        res_incorrect(conn);
    } else {
        if (admin_session) free(admin_session);
        admin_session = kqm_random_password(16);
        res_login(conn, admin_session);
    }

    admin_form_dtor(&form);
    return 1;
}
